/*  mainwindow.c
 *
 *  Main game routines
 *
 *  (c) 2009 Anton Olkhovik <ant007h@gmail.com>
 *
 *  This file is part of Mokomaze - labyrinth game.
 *
 *  Mokomaze is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Mokomaze is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Mokomaze.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <math.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_thread.h>
#include <SDL/SDL_ttf.h>
#include "types.h"
#include "accelerometers.h"
#include "paramsloader.h"
#include "vibro.h"


//#define BALL_MASS 1
#define GRAV_CONST 9.8*1.0
#define FRICT_COEF 0.20
#define TIME_QUANT 1.8
#define SPEED_TO_PIXELS 0.05
#define FORCE_TREASURE 0.055
#define BUMP_COEF 0.3

#define PROC_ACC_DATA_INTERVAL 20

double px,py;
double vx,vy;
double pr_px, pr_py;
double pr_vx, pr_vy;

double prev_px, prev_py; // for renderer

Config game_config;
Level* game_levels;
int game_levels_count;
int cur_level;

int fullscreen;

#define GAME_STATE_NORMAL   1
#define GAME_STATE_FAILED   2
#define GAME_STATE_WIN      3
int game_state;
int new_game_state;

SDL_mutex *mut;

void BumpVibrate(double speed)
{
    #define MAX_BUMP_SPEED 160.0
    #define MIN_BUMP_SPEED 45
    //#define MAX_BUMP_SPEED 160.0
    //#define VIB_TRESHHOLD 0.30
    double v = abs(speed);
    double k = v/MAX_BUMP_SPEED;
    if (k>1) k=1;
    //if (k>VIB_TRESHHOLD)
    if (v>MIN_BUMP_SPEED)
    {
        BYTE vlevel = (BYTE)(k*255);
        set_vibro(vlevel);
    }
}

void post_phys_res(double x, double y, double mm_vx, double mm_vy)
{
    SDL_mutexP(mut);
    px = x; py = y;
    SDL_mutexV(mut);

    vx = mm_vx; vy = mm_vy;
    pr_px = px; pr_py = py;
    pr_vx = vx; pr_vy = vy;
}

double tmp_px, tmp_py;
double tmp_vx, tmp_vy;
void post_temp_phys_res(double x, double y, double mm_vx, double mm_vy)
{
    if (x<game_config.ball_r)
    {
        BumpVibrate(mm_vx); //VIB_HOR
        x = game_config.ball_r;
        mm_vx = -mm_vx * BUMP_COEF;
    }
    if (x>game_config.wnd_w - game_config.ball_r)
    {
        BumpVibrate(mm_vx);
        x = game_config.wnd_w - game_config.ball_r;
        mm_vx = -mm_vx * BUMP_COEF;
    }
    if (y<game_config.ball_r)
    {
        BumpVibrate(mm_vy);
        y = game_config.ball_r;
        mm_vy = -mm_vy * BUMP_COEF;
    }
    if (y>game_config.wnd_h - game_config.ball_r)
    {
        BumpVibrate(mm_vy);
        y = game_config.wnd_h - game_config.ball_r;
        mm_vy = -mm_vy * BUMP_COEF;
    }

    tmp_px = x; tmp_py = y;
    tmp_vx = mm_vx; tmp_vy = mm_vy;
}

void apply_temp_phys_res()
{
    post_phys_res(tmp_px,tmp_py, tmp_vx,tmp_vy);
}

int line(double x0, double y0, double x1, double y1,    double vx0,double vy0, double vx1,double vy1);

Uint32 tout (Uint32 interval, void *param)
{

        if ( (game_state != GAME_STATE_NORMAL) || (!fullscreen) ) return PROC_ACC_DATA_INTERVAL;
        new_game_state = GAME_STATE_NORMAL;

//	printf("%f, %f, %f", getacx(), getacy(), getacz());

	double ax = getacx();
	double ay = getacy();
//	double m_ax= GRAV_CONST*ax;
//	double m_ay= GRAV_CONST*ay;

        double mid_px=px, mid_py=py;
        double mid_vx=vx, mid_vy=vy;

        double v = sqrt(mid_vx*mid_vx+mid_vy*mid_vy);
	double a = sqrt(ax*ax+ay*ay);
	if ((v > 0) || (a > FORCE_TREASURE))
	{
		mid_vx += ax*GRAV_CONST * TIME_QUANT;
		mid_vy += ay*GRAV_CONST * TIME_QUANT;
	}
	if (abs(mid_vx) > 0)
	{
		double dvx = abs( FRICT_COEF * GRAV_CONST*cos(asin(ax)) );
		if (mid_vx>0)
		{
			mid_vx-=dvx;
			if (mid_vx<0) mid_vx=0;
		}	
		else
		{
			mid_vx+=dvx;
			if (mid_vx>0) mid_vx=0;
		}	
	}
	if (abs(mid_vy) > 0)
	{
		double dvy = abs( FRICT_COEF * GRAV_CONST*cos(asin(ay)) );
		if (mid_vy>0)
		{
			mid_vy-=dvy;
			if (mid_vy<0) mid_vy=0;
		}	
		else
		{
			mid_vy+=dvy;
			if (mid_vy>0) mid_vy=0;
		}	
	}

	mid_px +=  (mid_vx * SPEED_TO_PIXELS);
	mid_py += -(mid_vy * SPEED_TO_PIXELS);

        if (!line(pr_px,pr_py, mid_px,mid_py,    pr_vx,pr_vy, mid_vx,mid_vy))
        {
            post_temp_phys_res(mid_px,mid_py, mid_vx,mid_vy);
            apply_temp_phys_res();
        }
	
	return PROC_ACC_DATA_INTERVAL;
}

double calcdist(double x1,double y1, double x2,double y2)
{
    return sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
}

double calclen(double x, double y)
{
    return sqrt(x*x + y*y);
}
//double calclen(DPoint p)
//{
//    return calclen(p.x, p.y);
//}

DPoint normalize(DPoint p)
{
    DPoint r;
    double len = calclen(p.x, p.y);
    r.x = p.x/len;
    r.y = p.y/len;
    return r;
}

int inbox(double x, double y, Box box)
{
    return ( (x>=box.x1) && (x<=box.x2) &&
             (y>=box.y1) && (y<=box.y2) );
}

int incircle(double x, double y,  double cx, double cy, double cr)
{
    return (calcdist(x,y, cx,cy) <= cr);
}


#define MAX_TRUEDIR_STEPS 5
int edgebump(int tx,int ty,   double x,double y,   double mm_vx,double mm_vy)
{
    Point touch;
    touch.x = tx; touch.y = ty;
    if (incircle(touch.x,touch.y, x,y,game_config.ball_r))
    {
        DPoint tmp_norm;
        tmp_norm.x = (x - touch.x);
        tmp_norm.y = (y - touch.y);
        DPoint norm = normalize(tmp_norm);
        mm_vy = -mm_vy;
        double dot_norm_vect = mm_vx*norm.x + mm_vy*norm.y;
        mm_vx = mm_vx - 2*norm.x*dot_norm_vect;
        mm_vy = mm_vy - 2*norm.y*dot_norm_vect;

        double vpr_x = -abs(dot_norm_vect)*norm.x;
        double vpr_y = -abs(dot_norm_vect)*norm.y;
        BumpVibrate(calclen(vpr_x, vpr_y));

        double dop_x = (1-BUMP_COEF) * vpr_x;
        double dop_y = (1-BUMP_COEF) * vpr_y;
        mm_vx = mm_vx + dop_x;
        mm_vy = mm_vy + dop_y;
        mm_vy = -mm_vy;

        x = touch.x + norm.x*(game_config.ball_r+0.10);
        y = touch.y + norm.y*(game_config.ball_r+0.10);

        post_temp_phys_res(x,y, mm_vx,mm_vy);
        return 1;
    }
    return 0;
}

#define HOLE_SCALE 1.00
int testbump(double x,double y,   double mm_vx,double mm_vy)
{
    
    Point final_hole = game_levels[cur_level].fins[0];
    double dist = calcdist(x,y, final_hole.x,final_hole.y);
    if (dist <= game_config.hole_r * HOLE_SCALE)
    {
        new_game_state = GAME_STATE_WIN;
        return 1;
    }

    for (int i=0; i<game_levels[cur_level].holes_count; i++)
    {
        Point hole = game_levels[cur_level].holes[i];
        Box boundbox;
        boundbox.x1 = hole.x - game_config.hole_r*HOLE_SCALE -1;
        boundbox.y1 = hole.y - game_config.hole_r*HOLE_SCALE -1;
        boundbox.x2 = hole.x + game_config.hole_r*HOLE_SCALE +1;
        boundbox.y2 = hole.y + game_config.hole_r*HOLE_SCALE +1;
//        if ( (abs(hole.x-x) <= game_config.hole_r*HOLE_SCALE) ||
//             (abs(hole.y-x) <= game_config.hole_r*HOLE_SCALE) )
        if (inbox(x,y, boundbox))
        {
            double dist = calcdist(x,y, hole.x,hole.y);
            if (dist <= game_config.hole_r * HOLE_SCALE)
            {
                new_game_state = GAME_STATE_FAILED;
                return 1;
            }
        }
    }


    int retval = 0;

    for (int i=0; i<game_levels[cur_level].boxes_count; i++)
    {
        Box box = game_levels[cur_level].boxes[i];
        Box boundbox;
        boundbox.x1 = box.x1 - game_config.hole_r*HOLE_SCALE -1;
        boundbox.y1 = box.y1 - game_config.hole_r*HOLE_SCALE -1;
        boundbox.x2 = box.x2 + game_config.hole_r*HOLE_SCALE +1;
        boundbox.y2 = box.y2 + game_config.hole_r*HOLE_SCALE +1;
        if (inbox(x,y, boundbox))
        {
            if (inbox(x,y-game_config.ball_r, box))
            {
                BumpVibrate(mm_vy);
                y = box.y2+game_config.ball_r+0.2;
                mm_vy = -mm_vy * BUMP_COEF;
                //post_phys_res(x,y, mm_vx,mm_vy);
                retval = 1;
                //return 1;
            }
            if (inbox(x,y+game_config.ball_r-0, box))
            {
                BumpVibrate(mm_vy);
                y = box.y1-game_config.ball_r -1.00;
                mm_vy = -mm_vy * BUMP_COEF;
                //post_phys_res(x,y, mm_vx,mm_vy);
                retval = 1;
                //return 1;
            }
            if (inbox(x-game_config.ball_r,y, box))
            {
                BumpVibrate(mm_vx);
                x = box.x2+game_config.ball_r+0.2;
                mm_vx = -mm_vx * BUMP_COEF;
                //post_phys_res(x,y, mm_vx,mm_vy);
                retval = 1;
                //return 1;
            }
            if (inbox(x+game_config.ball_r-0,y, box))
            {
                BumpVibrate(mm_vx);
                x = box.x1-game_config.ball_r -1.00;
                mm_vx = -mm_vx * BUMP_COEF;
                //post_phys_res(x,y, mm_vx,mm_vy);
                retval = 1;
                //return 1;
            }

            if (edgebump(box.x1,box.y1,  /*-1,-1,*/  x,y, mm_vx,mm_vy)) return 1;
            if (edgebump(box.x2,box.y1,  /* 1,-1,*/  x,y, mm_vx,mm_vy)) return 1;
            if (edgebump(box.x2,box.y2,  /* 1, 1,*/  x,y, mm_vx,mm_vy)) return 1;
            if (edgebump(box.x1,box.y2,  /*-1, 1,*/  x,y, mm_vx,mm_vy)) return 1;
        }

    }

    if (retval)
    {
        post_temp_phys_res(x,y, mm_vx,mm_vy);
    }

    return retval;
}

#define MAX_PHYS_ITERATIONS 10
int line(double x0, double y0, double x1, double y1,    double vx0,double vy0, double vx1,double vy1)
{

    double x=x0, y=y0;
    double mm_vx=vx1, mm_vy=vy1; //

    DPoint vec;
//    DPoint v_vec;
    vec.x = x1-x0;
    vec.y = y1-y0;
//    v_vec.x = vx1-vx0;
//    v_vec.y = vy1-vy0;
    DPoint norm_vec;
//    DPoint norm_v_vec;
    double len = calclen(vec.x, vec.y);
//    double vlen = calclen(v_vec.x, v_vec.y);

    norm_vec.x = vec.x/len;
    norm_vec.y = vec.y/len;
//    norm_v_vec.x = v_vec.x/vlen;
//    norm_v_vec.y = v_vec.y/vlen;
    double k=0;
    int muststop=0;    
    while (1)
    {
        k += 1;
        if (k>=len)
        {
            k=len;
            muststop=1;
        }
        x = x0 + norm_vec.x*k;
        y = y0 + norm_vec.y*k;
//        mm_vx = vx0 + norm_v_vec.x * vlen*(k/len);
//        mm_vy = vy0 + norm_v_vec.y * vlen*(k/len);
        if (testbump(x,y,  mm_vx,mm_vy))
        {
            int ko=0;
            while (new_game_state == GAME_STATE_NORMAL)
            {
                ko++;
                //TODO: lite testbump version
                int bump = testbump(tmp_px,tmp_py,  tmp_vx,tmp_vy);
                if ( (!bump) || (ko >= MAX_PHYS_ITERATIONS) ) break;
            }
            apply_temp_phys_res();
            game_state = new_game_state;
            return 1;
        }
        if (muststop) break;
    }

    return 0;

}

void InitState()
{
    px=game_levels[cur_level].init.x; py=game_levels[cur_level].init.y;
    vx=0; vy=0;

    pr_px=px; pr_py=py;
    pr_vx=0; pr_vy=0;

    prev_px=px; prev_py=py;
}

#define FONT_NAME "LiberationMono-Regular.ttf"
#define FIRST_TRY_DELAY 500
//#define NEXT_FRAME_DELAY 20000

void render_window(int start_level)
{
	SDL_Surface *screen;
	SDL_Event event;

        game_config = GetGameConfig();
        game_levels = GetGameLevels();
        game_levels_count = GetGameLevelsCount();

	// Load Pictures
	SDL_Surface *picture, *hole_pic, *fin_pic;
	picture = IMG_Load(MDIR "ball.gif");
        hole_pic = IMG_Load(MDIR "hole.gif");
        fin_pic = IMG_Load(MDIR "fin.gif");

        //== font and labels ===================================================
        TTF_Font* font = TTF_OpenFont(MFONTDIR FONT_NAME, 24);
        if (!font)
        {
            fprintf(stderr, "Main: can't load font '" MFONTDIR FONT_NAME "'. Exiting.\n");
            return;
        }
        SDL_Color fontColor = { 255, 255, 255 };
        SDL_Surface *levelTextSurface, *infoTextSurface;
        levelTextSurface = TTF_RenderText_Blended(font, "Level XX/YY", fontColor);
        infoTextSurface  = TTF_RenderText_Blended(font, "Touch the screen to continue", fontColor);
        SDL_Rect levelTextLocation = {  (game_config.wnd_w - levelTextSurface->w) / 2,
                                        levelTextSurface->h * 2,
                                        0, 0 };
        //== buttons ===========================================================
        SDL_Surface *back_pic, *forward_pic, *reset_pic;
        back_pic    = IMG_Load(MDIR "back.gif");
        forward_pic = IMG_Load(MDIR "forward.gif");
        reset_pic   = IMG_Load(MDIR "reset.gif");
        //== pos
        int btnw = back_pic->w;
        int btnh = back_pic->h;
        SDL_Rect gui_rect_1, gui_rect_2, gui_rect_3;
        gui_rect_1.y = gui_rect_2.y = gui_rect_3.y = levelTextSurface->h * (2*2 + 1);
        int btnspace = (game_config.wnd_w - btnw*3) / 4;
        gui_rect_1.x = btnspace*1 + btnw*0;
        gui_rect_2.x = btnspace*2 + btnw*1;
        gui_rect_3.x = btnspace*3 + btnw*2;
        //== for click detection
        Box gui_box_1, gui_box_2, gui_box_3;
        gui_box_1.x1 = gui_rect_1.x;        gui_box_1.y1 = gui_rect_1.y;
        gui_box_1.x2 = gui_rect_1.x + btnw; gui_box_1.y2 = gui_rect_1.y + btnh;
        gui_box_2.x1 = gui_rect_2.x;        gui_box_2.y1 = gui_rect_2.y;
        gui_box_2.x2 = gui_rect_2.x + btnw; gui_box_2.y2 = gui_rect_2.y + btnh;
        gui_box_3.x1 = gui_rect_3.x;        gui_box_3.y1 = gui_rect_3.y;
        gui_box_3.x2 = gui_rect_3.x + btnw; gui_box_3.y2 = gui_rect_3.y + btnh;
        //==
        SDL_Rect infoTextLocation  = {  (game_config.wnd_w - infoTextSurface->w) / 2,
                                        game_config.wnd_h/2 + levelTextSurface->h*(2*2 + 1)/2 + btnh/2  -  infoTextSurface->h/2,
                                        0, 0 };

        //== initializing window ===============================================
        fullscreen = 1;
        Uint32 sdl_flags = SDL_SWSURFACE;
        if (game_config.fullscreen) sdl_flags |= SDL_FULLSCREEN;
	screen = SDL_SetVideoMode(game_config.wnd_w, game_config.wnd_h, 16, sdl_flags);
	SDL_WM_SetCaption("Mokomaze", "Mokomaze");

        if (game_config.fullscreen)
        {
            SDL_ShowCursor(!fullscreen);
        }

	//SDL_Surface *desk_pic;
        //desk_pic = IMG_Load("desk.gif");

        /* Start the accelerometer thread */
	accelerometer_start();

        mut = SDL_CreateMutex();
        cur_level=start_level;
        InitState();
        game_state = GAME_STATE_NORMAL;
        SDL_TimerID main_timer = SDL_AddTimer(FIRST_TRY_DELAY, tout, NULL);

        int first_draw = 1;
	int done = 0;

	SDL_Rect box_rect;
	SDL_Rect ball_rect;
	SDL_Rect hole_rect;
	ball_rect.w=game_config.ball_r * 2; ball_rect.h=game_config.ball_r * 2;
	hole_rect.w=game_config.hole_r * 2; hole_rect.h=game_config.hole_r * 2;

        SDL_Rect desk_rect;
        desk_rect.x=0; desk_rect.y=0;
        desk_rect.w=game_config.wnd_w; desk_rect.h=game_config.wnd_h;

        Uint32 backColor = SDL_MapRGB(screen->format, 0, 0, 0);
        Uint32 boxColor  = SDL_MapRGB(screen->format, 100, 100, 100);

        Point mouse = {0, 0};

	while(!done) {

                int wasclick=0;

		while(SDL_PollEvent(&event))
                {
                    if (event.type == SDL_QUIT)
                    {
                        done=1;
                    }
                    if (event.type == SDL_MOUSEMOTION)
                    {
                        mouse.x = event.motion.x;
                        mouse.y = event.motion.y;
                    }
                    if (event.type == SDL_MOUSEBUTTONDOWN)
                    {
                        if (!fullscreen)
                        {
                            if (inbox(mouse.x,mouse.y, gui_box_1))
                            {
                                if (cur_level > 0)
                                {
                                    SDL_FillRect(screen, &desk_rect, backColor);
                                    cur_level--;
                                    InitState();
                                    game_state = GAME_STATE_NORMAL;
                                    first_draw=1;
                                    wasclick=1; //
                                }
                                continue;
                            }

                            if (inbox(mouse.x,mouse.y, gui_box_2))
                            {
                                if (cur_level < game_levels_count-1)
                                {
                                    SDL_FillRect(screen, &desk_rect, backColor);
                                    cur_level++;
                                    InitState();
                                    game_state = GAME_STATE_NORMAL;
                                    first_draw=1;
                                    wasclick=1; //
                                }
                                continue;
                            }

                            if (inbox(mouse.x,mouse.y, gui_box_3))
                            {
                                if (cur_level>0)
                                {
                                    SDL_FillRect(screen, &desk_rect, backColor);
                                    cur_level=0;
                                    InitState();
                                    game_state = GAME_STATE_NORMAL;
                                    first_draw=1;
                                    wasclick=1; //
                                }
                                continue;
                            }
                        }

                        fullscreen = !fullscreen;
                        if (!fullscreen)
                        {
                            wasclick=1;
                        }
                        else
                        {
                            SDL_FillRect(screen, &desk_rect, backColor);
                            first_draw=1;
                        }

                        if (game_config.fullscreen) 
                        {
                            SDL_WM_ToggleFullScreen(screen);
                            SDL_ShowCursor(!fullscreen);
                        }
                    }
		}

                if ((!fullscreen) && (!wasclick)) continue;

                SDL_mutexP(mut);
                int tk_px=px, tk_py=py;
                SDL_mutexV(mut);

                ball_rect.x=prev_px-game_config.ball_r; ball_rect.y=prev_py-game_config.ball_r;
		SDL_FillRect(screen, &ball_rect, backColor);

		//SDL_BlitSurface(desk_pic, NULL, screen, &desk_rect);

                for (int i=0; i<game_levels[cur_level].boxes_count; i++)
                {
                    box_rect.x=game_levels[cur_level].boxes[i].x1;
                    box_rect.y=game_levels[cur_level].boxes[i].y1;
                    box_rect.w=game_levels[cur_level].boxes[i].x2-box_rect.x;
                    box_rect.h=game_levels[cur_level].boxes[i].y2-box_rect.y;
                    SDL_FillRect(screen, &box_rect, boxColor);
                }

                for (int i=0; i<game_levels[cur_level].holes_count; i++)
                {
                    hole_rect.x=game_levels[cur_level].holes[i].x - game_config.hole_r;
                    hole_rect.y=game_levels[cur_level].holes[i].y - game_config.hole_r;
                    SDL_BlitSurface(hole_pic, NULL, screen, &hole_rect);
                }

                hole_rect.x=game_levels[cur_level].fins[0].x - game_config.hole_r;
                hole_rect.y=game_levels[cur_level].fins[0].y - game_config.hole_r;
                SDL_BlitSurface(fin_pic, NULL, screen, &hole_rect);

                //==================
                ball_rect.x=tk_px-game_config.ball_r; ball_rect.y=tk_py-game_config.ball_r;
		SDL_BlitSurface(picture, NULL, screen, &ball_rect);
                //updating the screen
                int min_px, max_px;
                int min_py, max_py;
                if (prev_px <= tk_px)
                {
                    min_px = prev_px;
                    max_px = tk_px;
                }
                else
                {
                    min_px = tk_px;
                    max_px = prev_px;
                }

                if (prev_py <= tk_py)
                {
                    min_py = prev_py;
                    max_py = tk_py;
                }
                else
                {
                    min_py = tk_py;
                    max_py = prev_py;
                }
                min_px -= game_config.ball_r;
                max_px += game_config.ball_r;
                min_py -= game_config.ball_r;
                max_py += game_config.ball_r;
                if (min_px<0) min_px=0;
                if (max_px>game_config.wnd_w) max_px=game_config.wnd_w;
                if (min_py<0) min_py=0;
                if (max_py>game_config.wnd_h) max_py=game_config.wnd_h;
                SDL_UpdateRect(screen, min_px, min_py, max_px-min_px, max_py-min_py);
                //---------------
                prev_px=tk_px; prev_py=tk_py;

                //== GUI =======================================================
                if ((wasclick) && (!fullscreen))
                {
                    char txt[32];
                    sprintf(txt, "Level %2d/%2d", cur_level+1, game_levels_count);
                    SDL_FreeSurface(levelTextSurface);
                    levelTextSurface = TTF_RenderText_Blended(font, txt, fontColor);
                    SDL_BlitSurface(levelTextSurface, NULL, screen, &levelTextLocation);
                    SDL_BlitSurface(infoTextSurface, NULL, screen, &infoTextLocation);
                    SDL_BlitSurface(back_pic, NULL, screen, &gui_rect_1);
                    SDL_BlitSurface(forward_pic, NULL, screen, &gui_rect_2);
                    SDL_BlitSurface(reset_pic, NULL, screen, &gui_rect_3);
                    SDL_Flip(screen);
                }

                // update the screen buffer
                if (first_draw)
                {
                    SDL_Flip(screen);
                    first_draw=0;
                }

                if (game_state == GAME_STATE_FAILED)
                {
                    SDL_FillRect(screen, &desk_rect, backColor);
                    InitState();
                    game_state = GAME_STATE_NORMAL;
                    first_draw=1;
                }

                if (game_state == GAME_STATE_WIN)
                {
                    SDL_FillRect(screen, &desk_rect, backColor);
                    if (cur_level == game_levels_count-1)
                    {
                        cur_level=0;
                    }
                    else
                    {
                        cur_level++;
                    }
                    InitState();
                    game_state = GAME_STATE_NORMAL;
                    first_draw=1;
                }

		//usleep(NEXT_FRAME_DELAY);
                SDL_Delay(game_config.f_delay);
                
	}

        SaveLevel(cur_level+1);

        SDL_FreeSurface(infoTextSurface);
        SDL_FreeSurface(levelTextSurface);
        TTF_CloseFont(font);

        SDL_RemoveTimer(main_timer);
        SDL_DestroyMutex(mut);

        accelerometer_stop();
        
}
